package com.winhc.bigdata.spark.ng.dynamic.handle

import com.winhc.bigdata.spark.implicits.Bool._
import com.winhc.bigdata.spark.ng.dynamic.NgCompanyRiskLevelType.NgCompanyRiskLevelType
import com.winhc.bigdata.spark.ng.dynamic._
import com.winhc.bigdata.spark.ng.dynamic.utils._
import com.winhc.bigdata.spark.utils.RegCapitalAmount

import scala.collection.mutable

/**
 * @author: XuJiakai
 * @date: 2021/6/23 10:43
 */
case class company_holder(is_inc: Boolean) extends NgCompanyDynamicHandle {
  override def flat_map: (ChangeExtract) => Seq[CompanyDynamicRecord] = (change_extract: ChangeExtract) => {
    val change_fields = change_extract.change_fields
    val tn = change_extract.tn
    val company_id = change_extract.company_id
    val company_name = change_extract.company_name
    val update_type = change_extract.update_type
    val old_data = change_extract.old_data
    val new_data = change_extract.new_data
    val biz_date = change_extract.biz_date
    val rowkey = change_extract.rowkey
    val update_time = change_extract.update_time


    def getRecordPro(dynamicCode: String
                     , name: String
                     , keyno: String
                     , label: String
                     , risk_level: NgCompanyRiskLevelType
                     , content: Seq[ChangeContent]
                    ): CompanyDynamicRecord = {
      val map = BusinessInfoDynamicInfoMap(label = label, content = content)

      val date = if (is_inc) update_time else biz_date
      CompanyDynamicRecord(
        id = CompanyDynamicUtils.generateId(rowkey, date, tn, dynamicCode)
        , association_entity_info = Seq(AssociationEntityInfo(keyno = keyno, name = name, risk_level = risk_level, rta_info = null))
        , rowkey = rowkey
        , tn = tn
        , update_type = update_type
        , dynamic_code = dynamicCode
        , dynamic_info = map.toMap()
        , agg_detail_text = null
        , agg_detail_rowkey = null
        , biz_time = biz_date
        , dynamic_time = date
        , update_time = update_time
        , create_time = update_time
      )
    }


    var list: mutable.Seq[CompanyDynamicRecord] = mutable.Seq.empty


    //股东出资变化
    def shareholderCapitalChange(name: String, keyno: String, before: String, after: String): Seq[CompanyDynamicRecord] = {
      val bool = RegCapitalAmount.isAdditionalShare(before, after)
      val flag = bool ? "1" | "0"
      //      val r = bool ? NgCompanyRiskLevelType.Positive | NgCompanyRiskLevelType.Caution
      val cdr = getRecordPro(
        dynamicCode = "101101"
        , keyno = company_id
        , name = company_name
        , label = "股东变化"
        , risk_level = NgCompanyRiskLevelType.Prompt
        , content = Seq(ChangeContent("101101"
          , before = Entity(name, keyno, info = before)
          , after = Entity(name, keyno, info = after)
          , Entity(name = company_name, keyno = company_id)
          , flag = flag
        ))
      )

      val cdr2 = getRecordPro(
        dynamicCode = "109003"
        , keyno = keyno
        , name = name
        , label = "对外投资"
        , risk_level = NgCompanyRiskLevelType.Prompt
        , content = Seq(ChangeContent("109003"
          , before = Entity(name = company_name, keyno = company_id, info = before)
          , after = Entity(name = company_name, keyno = company_id, info = after)
          , Entity(name = name, keyno = keyno)
          , flag = flag
        ))
      )


      Seq(cdr2, cdr)
    }

    //股东人员变化
    def shareholder(beforeName: String, beforeKeyno: String, afterName: String, afterKeyno: String): Seq[CompanyDynamicRecord] = {
      if (beforeName == null) {
        val cdr = getRecordPro(
          dynamicCode = "101102"
          , keyno = company_id
          , name = company_name
          , label = "股东变化"
          , risk_level = NgCompanyRiskLevelType.Prompt
          , content = Seq(ChangeContent("101102", before = null
            , after = Entity(afterName, afterKeyno)
            , Entity(name = company_name, keyno = company_id)))
        )

        val cdr2 = getRecordPro(
          dynamicCode = "109004"
          , keyno = afterKeyno
          , name = afterName
          , label = "对外投资"
          , risk_level = NgCompanyRiskLevelType.Prompt
          , content = Seq(ChangeContent("109004", before = null
            , after = Entity(name = company_name, keyno = company_id)
            , Entity(afterName, afterKeyno)))
        )
        Seq(cdr, cdr2)
      } else {
        val cdr = getRecordPro(
          dynamicCode = "101102"
          , keyno = company_id
          , name = company_name
          , label = "股东变化"
          , risk_level = NgCompanyRiskLevelType.Prompt
          , content = Seq(ChangeContent("101102", before = Entity(beforeName, beforeKeyno)
            , after = null
            , Entity(name = company_name, keyno = company_id)))
        )

        val cdr2 = getRecordPro(
          dynamicCode = "109004"
          , keyno = beforeKeyno
          , name = beforeName
          , label = "对外投资"
          , risk_level = NgCompanyRiskLevelType.Prompt
          , content = Seq(ChangeContent("109004", before = Entity(name = company_name, keyno = company_id)
            , after = null
            , Entity(beforeName, beforeKeyno)))
        )
        Seq(cdr, cdr2)
      }

    }

    //股东新增对外投资
    def holderNew(name: String, keyno: String, companyName: String, companyId: String): Seq[CompanyDynamicRecord] = {
      val cdr = getRecordPro(
        dynamicCode = "109004"
        , keyno = keyno
        , name = name
        , label = "对外投资"
        , risk_level = NgCompanyRiskLevelType.Prompt
        , content = Seq(ChangeContent("109004", before = null
          , after = Entity(companyName, companyId)
          , Entity(name = name, keyno = keyno)))
      )
      Seq(cdr)
    }


    update_type match {
      case "insert" => {
        //新增股东
        val n_company_id = new_data("company_id")
        val n_company_name = new_data("company_name")
        val n_holder_name = new_data("holder_name")
        val n_holder_id = new_data("holder_id")

        list = list ++ shareholder(null, null, n_holder_name, n_holder_id)
      }
      case "create" => {
        //公司新成立
        val n_holder_name = new_data("holder_name")
        val n_holder_id = new_data("holder_id")
        list = list ++ holderNew(n_holder_name, n_holder_id, company_name, company_id)
      }

      case "update" => {
        //股东出资额发变化
        if (change_fields.contains("amount")) {
          val old_amount = old_data("amount")
          val new_amount = new_data("amount")

          val n_company_id = new_data("company_id")
          val n_company_name = new_data("company_name")
          val n_holder_name = new_data("holder_name")
          val n_holder_id = new_data("holder_id")

          list = list ++ shareholderCapitalChange(n_holder_name, n_holder_id, old_amount, new_amount)
        }
      }

      case "deleted" => {
        //新增股东
        val n_company_id = new_data("company_id")
        val n_company_name = new_data("company_name")
        val n_holder_name = new_data("holder_name")
        val n_holder_id = new_data("holder_id")
        list = list ++ shareholder(n_holder_name, n_holder_id, null, null)
      }
    }
    list
  }

/*

  override def group_by_key: CompanyDynamicRecord => String = (cdr: CompanyDynamicRecord) => {
    val info = cdr.association_entity_info.filter(_.keyno != null).find(r => r.keyno.length == 32).orNull
    if (info == null) {
      cdr.id
    } else {
      info.keyno
    }
  }

  override def group_by_flat_map: Seq[CompanyDynamicRecord] => Seq[CompanyDynamicRecord] = (seq: Seq[CompanyDynamicRecord]) => {
    seq
  }
*/

}
