package com.winhc.bigdata.spark.ng.dynamic.handle

import com.winhc.bigdata.spark.ng.dynamic.NgCompanyRiskLevelType.NgCompanyRiskLevelType
import com.winhc.bigdata.spark.ng.dynamic._
import com.winhc.bigdata.spark.ng.dynamic.utils.{BusinessInfoDynamicInfoMap, ChangeContent, CompanyDynamicUtils, Entity}

import scala.collection.mutable

/**
 * @author: XuJiakai
 * @date: 2021/7/26 16:15
 */
case class company_staff(is_inc: Boolean) extends NgCompanyDynamicHandle {
  override def flat_map: ChangeExtract => Seq[CompanyDynamicRecord] = (change_extract: ChangeExtract) => {
    val change_fields = change_extract.change_fields
    val tn = change_extract.tn
    val company_id = change_extract.company_id
    val company_name = change_extract.company_name
    val update_type = change_extract.update_type
    val old_data = change_extract.old_data
    val new_data = change_extract.new_data
    val biz_date = change_extract.biz_date
    val rowkey = change_extract.rowkey
    val update_time = change_extract.update_time


    def getRecord(dynamic_code: String
                  , name: String
                  , keyno: String
                  , description: String
                  , risk_level: NgCompanyRiskLevelType
                  , dynamic_info: Map[String, Any]
                 ): CompanyDynamicRecord = {
      val date = if (is_inc) update_time else biz_date
      CompanyDynamicRecord(
        id = CompanyDynamicUtils.generateId(rowkey, date, tn, dynamic_code)
        , association_entity_info = Seq(AssociationEntityInfo(keyno = keyno, name = name, risk_level = risk_level, rta_info = null))
        , rowkey = rowkey
        , tn = tn
        , update_type = update_type
        , dynamic_code = dynamic_code
        , dynamic_info = dynamic_info
        , agg_detail_text = null
        , agg_detail_rowkey = null
        , biz_time = biz_date
        , dynamic_time = date
        , update_time = update_time
        , create_time = update_time
      )
    }


    def getRecordPro(dynamicCode: String
                     , name: String
                     , keyno: String
                     , label: String
                     , risk_level: NgCompanyRiskLevelType
                     , content: Seq[ChangeContent]
                    ): CompanyDynamicRecord = {
      val map = BusinessInfoDynamicInfoMap(label = label, content = content)

      val date = if (is_inc) update_time else biz_date
      CompanyDynamicRecord(
        id = CompanyDynamicUtils.generateId(rowkey, date, tn, dynamicCode)
        , association_entity_info = Seq(AssociationEntityInfo(keyno = keyno, name = name, risk_level = risk_level, rta_info = null))
        , rowkey = rowkey
        , tn = tn
        , update_type = update_type
        , dynamic_code = dynamicCode
        , dynamic_info = map.toMap()
        , agg_detail_text = null
        , agg_detail_rowkey = null
        , biz_time = biz_date
        , dynamic_time = date
        , update_time = update_time
        , create_time = update_time
      )
    }


    var list: mutable.Seq[CompanyDynamicRecord] = mutable.Seq.empty

    //新增主要成员
    def staffNew(staffName: String, staffId: String, staffType: String): Seq[CompanyDynamicRecord] = {
      val cdr = getRecordPro(
        dynamicCode = "109005"
        , keyno = company_id
        , name = company_name
        , label = "主要成员"
        , risk_level = NgCompanyRiskLevelType.Prompt
        , content = Seq(ChangeContent("109005", before = null
          , after = Entity(name = staffName, keyno = staffId, info = staffType)
          , Entity(company_name, company_id)))
      )

      val cdr2 = getRecordPro(
        dynamicCode = "109005"
        , keyno = staffId
        , name = staffName
        , label = "任职变化"
        , risk_level = NgCompanyRiskLevelType.Prompt
        , content = Seq(ChangeContent("109005", before = null
          , after = Entity(name = company_name, keyno = company_id, info = staffType)
          , Entity(staffName, staffId)))
      )
      Seq(cdr, cdr2)
    }

    //公司新成立
    def staffCreate(staffName: String, staffId: String, staffType: String): Seq[CompanyDynamicRecord] = {
      val cdr = getRecordPro(
        dynamicCode = "109005"
        , keyno = staffId
        , name = staffName
        , label = "任职变化"
        , risk_level = NgCompanyRiskLevelType.Prompt
        , content = Seq(ChangeContent("109005", before = null
          , after = Entity(name = company_name, keyno = company_id, info = staffType)
          , Entity(staffName, staffId)))
      )
      Seq(cdr)
    }

    //职位变化
    def staffTypeChange(staffName: String, staffId: String, beforeType: String, afterType: String): Seq[CompanyDynamicRecord] = {

      val cdr = getRecordPro(
        dynamicCode = "101301"
        , keyno = company_id
        , name = company_name
        , label = "主要成员"
        , risk_level = NgCompanyRiskLevelType.Prompt
        , content = Seq(ChangeContent("101301"
          , before = Entity(staffName, staffId, info = beforeType)
          , after = Entity(staffName, staffId, info = afterType)
          , Entity(name = company_name, keyno = company_id)
        ))
      )

      val cdr2 = getRecordPro(
        dynamicCode = "109005"
        , keyno = staffId
        , name = staffName
        , label = "任职变化"
        , risk_level = NgCompanyRiskLevelType.Prompt
        , content = Seq(ChangeContent("109005"
          , before = Entity(name = company_name, keyno = company_id, info = beforeType)
          , after = Entity(name = company_name, keyno = company_id, info = afterType)
          , Entity(name = staffName, keyno = staffId)
        ))
      )
      Seq(cdr, cdr2)
    }


    //主要成员移除
    def staffDeleted(staffName: String, staffId: String, staffType: String): Seq[CompanyDynamicRecord] = {
      val cdr = getRecordPro(
        dynamicCode = "109005"
        , keyno = company_id
        , name = company_name
        , label = "主要成员"
        , risk_level = NgCompanyRiskLevelType.Prompt
        , content = Seq(ChangeContent("109005", before = Entity(name = staffName, keyno = staffId, info = staffType)
          , after = null
          , Entity(company_name, company_id)))
      )

      val cdr2 = getRecordPro(
        dynamicCode = "109005"
        , keyno = staffId
        , name = staffName
        , label = "任职变化"
        , risk_level = NgCompanyRiskLevelType.Prompt
        , content = Seq(ChangeContent("109005", before = Entity(name = company_name, keyno = company_id, info = staffType)
          , after = null
          , Entity(staffName, staffId)))
      )
      Seq(cdr, cdr2)
    }


    update_type match {
      case "insert" => {
        //新增主要成员
        val n_staff_name = new_data("staff_name")
        val n_hid = new_data("hid")
        val n_staff_type = new_data("staff_type")

        list = list ++ staffNew(n_staff_name, n_hid, n_staff_type)
      }
      case "create" => {
        //公司新成立
        val n_staff_name = new_data("staff_name")
        val n_hid = new_data("hid")
        val n_staff_type = new_data("staff_type")

        list = list ++ staffCreate(n_staff_name, n_hid, n_staff_type)
      }

      case "update" => {
        //主要成员职位变化
        val n_staff_name = new_data("staff_name")
        val n_hid = new_data("hid")
        val n_staff_type = new_data("staff_type")
        val o_staff_type = old_data("staff_type")

        list = list ++ staffTypeChange(n_staff_name, n_hid, o_staff_type, n_staff_type)
      }

      case "deleted" => {
        //移除主要成员
        val n_staff_name = new_data("staff_name")
        val n_hid = new_data("hid")
        val n_staff_type = new_data("staff_type")

        list = list ++ staffDeleted(n_staff_name, n_hid, n_staff_type)
      }
    }
    list
  }
/*
  override def group_by_key: CompanyDynamicRecord => String = (cdr: CompanyDynamicRecord) => {
    val info = cdr.association_entity_info.filter(_.keyno != null).find(r => r.keyno.length == 32).orNull
    if (info == null) {
      cdr.id
    } else {
      info.keyno
    }
  }


  override def group_by_flat_map: Seq[CompanyDynamicRecord] => Seq[CompanyDynamicRecord] = (seq: Seq[CompanyDynamicRecord]) => {
    seq
  }*/
}
